/* This demo shows the use of event callbacks and form callbacks

   Written by Mark Overmars

   last changed: 29 Apr 1991
*/

#include <stdio.h>
#include <stdlib.h>
#include <gl/gl.h>
#include <gl/device.h>
#include "forms.h"

/*------------ Color Part --------------------*/

FL_FORM *colorform;
FL_OBJECT *redsl, *greensl, *bluesl, *colorobj;

short r= 128, g= 128, b= 128 ;

void color_callback(FL_OBJECT *obj)
/* color form callback routine */
{
  r = (short) (255.0*fl_get_slider_value(redsl));
  g = (short) (255.0*fl_get_slider_value(greensl));
  b = (short) (255.0*fl_get_slider_value(bluesl));
  fl_mapcolor(300,r,g,b);
  fl_redraw_object(colorobj);
}

void create_colorform()
/* creates the color form */
{
  colorform = fl_bgn_form(FL_UP_BOX,400.0,300.0);
  redsl = fl_add_slider(FL_VERT_SLIDER,20.0,40.0,40.0,240.0,"red");
    fl_set_object_color(redsl,47,1);
  greensl = fl_add_slider(FL_VERT_SLIDER,80.0,40.0,40.0,240.0,"green");
    fl_set_object_color(greensl,47,2);
  bluesl = fl_add_slider(FL_VERT_SLIDER,140.0,40.0,40.0,240.0,"blue");
    fl_set_object_color(bluesl,47,4);
  colorobj = fl_add_box(FL_UP_BOX,200.0,40.0,180.0,200.0,"Color");
    fl_set_object_color(colorobj,300,300);
    fl_set_object_lsize(colorobj,FL_LARGE_FONT);
    fl_set_object_align(colorobj,FL_ALIGN_TOP);
  fl_end_form();
}

void init_colorpart()
/* initializes the color part */
{
  create_colorform();
  fl_set_form_call_back(colorform,color_callback);
  fl_show_form(colorform,FL_PLACE_SIZE,FALSE,NULL);
  fl_mapcolor(300,r,g,b);
  fl_redraw_object(colorobj);
}


/*--------------- Control Part -------------------*/

FL_FORM *controlform;
FL_OBJECT *squareobj, *circleobj, *exitobj, * clearobj, *sizeobj, *triangleobj;

int curobj = 1;
int cursize = 20;

void clearit();

void control_callback(FL_OBJECT *obj)
/* control form callback routine */
{
  if (obj == squareobj) curobj = 1;
  else if (obj == circleobj) curobj = 2;
  else if (obj == triangleobj) curobj = 3;
  else if (obj == sizeobj) cursize = (int)(40.0 * fl_get_slider_value(sizeobj));
  else if (obj == clearobj) clearit();
  else if (obj == exitobj) exit(0);
}

void create_controlform()
{
  controlform = fl_bgn_form(FL_UP_BOX,260.0,230.0);
  fl_bgn_group();
  squareobj = fl_add_button(FL_RADIO_BUTTON,20.0,150.0,60.0,60.0,"@square");
    fl_set_object_lcol(squareobj,3);
  circleobj = fl_add_button(FL_RADIO_BUTTON,20.0,90.0,60.0,60.0,"@circle");
    fl_set_object_lcol(circleobj,3);
  triangleobj = fl_add_button(FL_RADIO_BUTTON,20.0,30.0,60.0,60.0,"@8>");
    fl_set_object_lcol(triangleobj,3);
  fl_end_group();
  exitobj = fl_add_button(FL_NORMAL_BUTTON,160.0,30.0,80.0,30.0,"Exit");
  clearobj = fl_add_button(FL_NORMAL_BUTTON,160.0,180.0,80.0,30.0,"Clear");
  sizeobj = fl_add_slider(FL_VERT_SLIDER,100.0,30.0,40.0,180.0,"size");
  fl_end_form();
}

void init_controlpart()
/* initializes the control part */
{
  create_controlform();
  fl_set_form_call_back(controlform,control_callback);
  fl_set_button(squareobj,1);
  fl_show_form(controlform,FL_PLACE_SIZE,FALSE,NULL);
}

/*--------------- Main part -------------------*/

typedef struct{
  int type, r, g, b, x, y, size;
} OBJ;

OBJ ob[10000];
int onumb = 0;

void drawit(OBJ ob)
{
  RGBcolor(ob.r,ob.g,ob.b);
  if (ob.type == 1)
    rectfi(ob.x-ob.size,ob.y-ob.size,ob.x+ob.size,ob.y+ob.size);
  else if (ob.type == 2)
    circfi(ob.x,ob.y,ob.size);
  else if (ob.type == 3)
  {
    pmv2i(ob.x-ob.size,ob.y-ob.size);
    pdr2i(ob.x+ob.size,ob.y-ob.size);
    pdr2i(ob.x,ob.y+ob.size);
    pclos();
  }
}

void drawobject()
/* draws a particular object */
{
  long xx,yy;
  getorigin(&xx,&yy);
  ob[onumb].x = (int) (getvaluator(MOUSEX) - xx);
  ob[onumb].y = (int) (getvaluator(MOUSEY) - yy);
  ob[onumb].r = r;
  ob[onumb].g = g;
  ob[onumb].b = b;
  ob[onumb].type = curobj;
  ob[onumb].size = cursize;
  drawit(ob[onumb]);
  onumb++;
}

void redrawit()
{
  int i;
  reshapeviewport();
  RGBcolor(0,0,0);
  clear();
  for (i=0; i<onumb; i++) drawit(ob[i]);
}

void clearit()
{
  onumb = 0;
  redrawit();
}

void main_callback(short dev, short val)
/* event callback routine */
{
  if (dev == REDRAW)
    redrawit();
  else if (dev == MOUSE3 && val == 1)
    drawobject();
  else if (dev == ESCKEY)
    exit(0);
}

void init_mainpart()
{
  prefposition(100,600,400,900);
  winopen("Drawing");
  RGBmode();
  gconfig();
  fl_qdevice(MOUSE3);
  fl_qdevice(ESCKEY);
  fl_set_event_call_back(main_callback);
  RGBcolor(0,0,0);
  clear();
}

/*---------------------------------------*/

main()
{
  fl_init();
  init_colorpart();
  init_controlpart();
  init_mainpart();

  fl_do_forms();
}
